# This file should be required

$file_cnt = 0;
$sock_cnt = 0;
$sync_cnt = 0;

@id_list = ();        # List of all ids currently in use

##################################################################
#
# Global array id_info
#
# ,type  :  client | server
# ,port  :  port of server
#
# ,pid   :  process id of server (server only)
# ,conn  :  connection to talk to this item on
#
# $file_cnt : Keep a file number for unique file ids.
# @id_list  : Keep a list of all ids in use.
#
##################################################################

sub do_play {

    my($file) = @_;
    my($fail,$ret) = (0,1);
    my($try) = (0);
    my(@my_cmmds);
    my($skip) = (0);

    open(PLAY,$file) || return 0;

    while (<PLAY>){

        chop;
        $skip=1,next if ( /^\s*#/ );

        @cmmds = split;
        $skip=1, next if (@cmmds == 0);

        @my_cmmds = @cmmds;
        print "@cmmds\n" if ($verbose);

        $cmmd = shift(@cmmds);

        $ret = &cmmd(@cmmds),   next if ($cmmd eq 'cmmd');
        $ret = &dump(@cmmds),   next if ($cmmd eq 'dump');
        $ret = &kill(@cmmds),   next if ($cmmd eq 'kill');
        $ret = &ping(@cmmds),   next if ($cmmd eq 'ping');
        $ret = &my_slp(@cmmds), next if ($cmmd eq 'sleep');
        $ret = &start(@cmmds),  next if ($cmmd eq 'start');
        $ret = &verify(@cmmds), next if ($cmmd eq 'verify');

        print "Command $cmmd is not implemented.\n";
    } continue {
        $fail = 1 if (! $ret);
        if ((! $ret) && (! $skip)){
            print "command '@my_cmmds' failed.\n";
            $fail = 1;
        }
        $skip = 0;
    }

    close PLAY;
   
    # Make sure all remaining clients/servers are dead;
    &kill(@id_list);

    return $fail;
}

#################################

sub cmmd {

    my($clnt,@ccmd) = @_;
    my($oldfh);

    return 0 if ($id_info{"$clnt,type"} ne 'c');

    $oldfh = select($id_info{"$clnt,conn"});
    print "@ccmd\n";
    select($oldfh);

    return(1);
}

#################################

sub dump {

    my($clnt,@files) = @_;
    my($retval) = (1);
    my($file);
    my($oldfh);

    return 0 if ($id_info{"$clnt,type"} ne 'c');

    foreach $file (@files){

        if (! open(DUMP,$file)){
            print "dump: Could not open file $file.\n";
            $retval = 0;
            next;
        }

        $oldfh = select($id_info{"$clnt,conn"});
        print <DUMP>;
        close DUMP;
        select($oldfh);
    }

    return $retval;
}

#################################

sub kill {

    my(@ids) = @_;
    my($fid,$pid);
    my($ret) = (1);
    my($j);

    foreach $id (@ids){

        if (! $id_info{"$id,type"}){
            $ret = 0;
            next;
        }

        $fid = $id_info{"$id,conn"};

        if ($id_info{"$id,type"} eq 's'){
            $pid = $id_info{"$id,pid"};
            kill 15, ($pid);       # SIGTERM
        } else {
            print $fid "quit.";
        }

        close $fid;
        $id_info{"$id,type"} = '';
        $id_info{"$id,conn"} = '';

        if ($id eq $id_list[0]){
            # The usual case
            shift @id_list;
        } else {
            # Should really remove from list, but its not worth it.
        }
    }

    return $ret;
}

#################################

sub ping {

    my(@ids) = @_;
    my($id,$ret);

    foreach $id (@ids){

        if ($id_info{"$id,type"} eq 'c'){
            &verify($id);        # Throw away all waiting input
            &cmmd($id,"sync($sync_cnt).");
            $ret = &verify($id,'local',"SYNC:$sync_cnt");
            &cmmd($id,"yield.");
            &verify($id,'local',"OKAY:");
            print "$id gave incorrect response to ping.\n" if (! $ret);
            $sync_cnt++;
            return $ret;
        } else {
            # Should really connect to server and do a sync or something
            return 1;
        }
    }
}

#######################################################
# My version of sleep, don't disturb the alarm too much
#
sub my_slp {

    my($secs) = @_;
    my($alrm_secs) = alarm 0;

    sleep $secs;

    # Restore script's lifetime and signal handler.
    &set_max_time($alrm_secs);
    return 1;
}

#################################

sub start {

    my($ctype,$serv_id,$clnt_id) = @_;
    my($ch,$my_id,$fid);
    my($pid,$port) = (0,0);
    my($coral_cmmd, $sock);
    my($name,$aliases,$type,$len,$addr,$hostname,$proto);
    my($this,$that);
    my($oldfh);

    $ch = substr($ctype,0,1); 

    if (($ch eq 's') || ($ch eq 'S')){
        $my_id = $serv_id;
        $ctype = 's';
        $port = $clnt_id if ($clnt_id);
    } elsif (($ch eq 'c') || ($ch eq 'C')){
        $my_id = $clnt_id;
        $ctype = 'c';
    }

    return 0 if (! $my_id);

    &kill($my_id) if ($id_info{"$my_id,type"});

    $id_info{"$my_id,type"} = $ctype;

    if ($clnt_id && ($ctype eq 'c')){

        # Start up a client
        if (! ($id_info{"$serv_id,type"} eq 's')){
            print "$serv_id is not a valid server id.\n";
            $id_info{"$my_id,type"} = '';
            return 0;
        }

        $port = $id_info{"$serv_id,port"};
        $AF_INET = 2;
        $SOCK_STREAM = 1;

        $SIG{'INT'} = 'dokill';

        sub dokill {
            kill 9, $child if $child;
        }

        $sockaddr = 'S n a4 x8';

        ($name,$aliases,$proto) = getprotobyname('tcp');
        chop($hostname = `hostname`);
        ($name,$aliases,$type,$len,$addr) = gethostbyname($hostname);

        $this = pack($sockaddr,$AF_INET,0,$addr);
        $that = pack($sockaddr,$AF_INET,$port,$addr);

        $sock = "SOCK${sock_cnt}";
        $sock_cnt++;

        die $! if (! socket($sock, $AF_INET, $SOCK_STREAM, $proto));
        die $! if (! bind($sock, $this));

        die $! if (! connect($sock,$that));

        # set socket to be unbuffered
        $oldfh = select($sock); $|=1; select($oldfh);

        $id_info{"$my_id,conn"} = $sock;

    } else {

        # Start up a server
        $fid = "FILE${file_cnt}";
        $file_cnt++;

        if ($port){
           $coral_cmmd = "$coral -s -p$port |";
        } else {
           $coral_cmmd = "$coral -s |";
        }

        retry: {
            if (! open( $fid, $coral_cmmd) ){
                print "Could not start coral server.\n";
                $id_info{"$serv_id,type"} = '';
                return(0);
            }
    
            chop($serv_info=<$fid>);
    
            if ($serv_info=~ /\[(\d+)\]: Coral Server started on port (\d+)/){
                $pid = $1;
                $port = $2;
            } else {
                # Could not bind local address?
                $try++; 
                if ($try > 4){
                    print "Could not start coral server";
                    print " (probably couldn't bind to local address)\n";
                    $id_info{"$serv_id,type"} = '';
                    return(0);
                } else {
                    &my_slp(1);
                    redo retry;
                }
            }
        }

        $id_info{"$serv_id,conn"} = $fid;
        $id_info{"$serv_id,pid"} = $pid;
        $id_info{"$serv_id,port"} = $port;
    }

  
    # Save all ids 
    push(@id_list,$my_id); 
    return(1);
}

#################################

sub read_avail {
    my($fh) = @_;
    my($num);

    $rin = '';
    vec($rin,fileno($fh),1) = 1;

    $num = select($rout=$rin,undef,undef,10); 
}

sub more_avail {
    my($fh) = @_;
    my($num);
    $num = select($rout=$rin,undef,undef,2); 
}

sub consume {

    my($conn) = @_;
    my($line);

    $line=<$conn> while (&more_avail);
}

#################################

sub verify {

    my($clnt,$file,@cmmd) = @_;
    my($conn,$line);
    my($i,$j,@words);
    my($retval) = (1);
    my($create);

    return 0 if (! $id_info{"$clnt,type"} eq 'c');

    $conn = $id_info{"$clnt,conn"};

    if (! &read_avail($conn)){
        print "verify: No info available on server socket.\n" if ($file);
        return 0;
    }

    if ($file eq 'local'){

        chop($line=<$conn>);
        @words = split(/\s+/,$line);
               
        foreach $i (@cmmd){
            
            $j = shift(@words);
            if ($i ne $j){
                $retval = 0;
                next;
            }
        }

    } elsif ($file) {

        # verify the file (or create it)
        if (! open(VERFY,$file)){
            $create = 1;
            open(VERFY,"> $file") ||
                    die "verify: Could not write to file: $file\n";

            print "verify: Creating non-existant output file: $file\n";

            while( &more_avail){

                chop($line=<$conn>);
                print VERFY "$line\n";
            }
            close VERFY;

        } else {

            while(<VERFY>){
                return 0 if (! &more_avail);

                chop;
                chop($line=<$conn>);

                if ($line ne $_){
                    &consume($conn);
                    return 0;
                }
            }
        }

    } else {
        # consume all available input
        &consume($conn);
    }

    return 1;
}

#################################

sub allow_serv_quit {

     # Watch for ^C and other requests to quit.
    $SIG{'HUP'}  = 'serv_quit_handler';
    $SIG{'INT'}  = 'serv_quit_handler';
    $SIG{'TERM'} = 'serv_quit_handler';
}

############################

sub serv_quit_handler {

    print "\n\nSignal requesting quit recieved. - quitting.\n\n";

    # Try to clean up a little
    
    # Make sure all remaining clients/servers are dead;
    kill(@id_list);

    &core_exist;

    if ((-d 'pgms') && (-f 'pgms/Makefile')){
        chdir('pgms');
        &make('clean');
    }

    exit 0;
}

############################

# So that require succeed.
1;
