#include "sybase.h"
#include "RdbSybaseProcess.h"
#include "RdbSybaseDatabase.h"
#include "Log.h"


int RdbSybaseProcessPool::total_inuse = 0;
int RdbSybaseProcessPool::limit = 0;


RdbSybaseProcessPool::RdbSybaseProcessPool(const String newDbName,
					   const String userid,
					   const String password,
					   const String newServerName) 
: dbName(newDbName),
  serverName(newServerName)
{
    if( limit == 0 )
      limit = dbgetmaxprocs();
    login = dblogin();
    assert(login);

    char *tmp = (char *) ((const char *) userid);
    DBSETLUSER(login, tmp);
    tmp = (char *) ((const char *) password);
    DBSETLPWD(login, tmp);
    DBSETLAPP(login, "Rdb Interface");
}


RdbSybaseProcessPool::~RdbSybaseProcessPool()
{
    assert( inuseProc.Size() == 0 ); // all processes should be closed!
    int avail = availableProc.Size();
    for( int i = avail -1 ; i >= 0 ; i-- ) {
	dbclose(availableProc.Delete(i));
    }
    dbloginfree(login);
}


RdbError::ErrorCode RdbSybaseProcessPool::AllocateProcess(DBPROCESS*& dbproc,
							  int& procid)
{
    RETCODE rc;
    dbproc = NULL;
    procid = -1;
    int avail = availableProc.Size();
    if( avail > 0 ) {
        dbproc = availableProc.Delete(avail-1);
    } else {			// none available, create a new one
        if( total_inuse == limit ) {
            LOG("Used all dbprocs, making room for more\n");
            limit += 10;
            dbsetmaxprocs(limit); // make room for some more
        }

        char *tmp = (char *) ((const char *) serverName);
        dbproc = dbopen(login, tmp);
        if( dbproc == NULL ) {
            LOG("Could not open connection!!\n");
            return RdbError::SQL_ERROR;
        }

        tmp = (char *) ((const char *) dbName);
        rc = dbuse(dbproc, tmp);
        if( rc != SUCCEED ) {
            LOG("Could not open database " << dbName << "!!\n");
            return RdbError::SQL_ERROR;
        }
    }
    total_inuse++;
    procid = inuseProc.Append(dbproc);
    return RdbError::NO_ERROR;
}


RdbError::ErrorCode RdbSybaseProcessPool::ReleaseProcess(int procid)
{
    total_inuse--;
    availableProc.Append( inuseProc.Delete(procid) );
    return RdbError::NO_ERROR;
}


//---------------------------------------------------------------------------


RdbSybaseProcess::RdbSybaseProcess(RdbSybaseDatabase& newDatabase)
: database(newDatabase)
{
    dbproc = NULL;
    database.processPool->AllocateProcess(dbproc, procid);
}


RdbSybaseProcess::~RdbSybaseProcess()
{
    database.processPool->ReleaseProcess(procid);
}
