
//**************************************************************************
//    CoralSock.h
//
//    The CoralSock Class - A class for providing socket operations for
//        Coral programs.
//
//**************************************************************************

#ifndef _CoralSock_H_
#define _CoralSock_H_

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>   
#include <unistd.h>

#include "Identify.h"    // To define the system type

  // Why are there no networking prototypes? 
extern "C" {

int socket(int domain, int type, int protocol);
int listen(int s, int backlog);
int shutdown(int s, int how);

#if defined(HPUX)

int bind(int s, const void *name, int namelen);
int accept(int s, void *addr, int *addrlen);
int connect(int s, const void *name, int namelen);
int gethostname(char *name, size_t namelen);


#elif defined(SUN)

int bind(int s, struct sockaddr *name, int namelen);
int accept(int s, struct sockaddr *addr, int *addrlen);
int connect(int s, struct sockaddr *name, int namelen);
int gethostname(char *name, int namelen);
int select(int width, fd_set *readfds, fd_set *writefds,
         fd_set *exceptfds, struct timeval *timeout); 

#endif

}
         
//**************************************************************************

            // default PORT, unless env var CORALPORT exists 
#define CORAL_PORT   6006

            // Port used by explain to talk to server 
#define EXPLAIN_PORT 6007

            // default SERVER, unless env var CORALSERVER exists 
#define CORAL_SERVER "fontina"

#define HOSTNAME_LENGTH 100

  // The types of sockets
  //
  // ServerType is a socket used to accept connections at a well-known
  //     port. 
  // ClientType is a socket used by clients talking to the server, as
  //     well as servers talking to the clients.
enum SockType { ServerType, ClientType };

//**************************************************************************

class CoralSock {

protected:
  SockType socketType;        // The type of CoralSock this is

    // A simple CoralSock Constructor
  CoralSock(int fd, SockType stype) {
    socketfd   = fd;
    socketType = stype;
    in = out = (FILE *) NULL;
  }

    // A private function to assist the constructors
  CoralSock *open_client(CoralSock *cs, char *HostName, unsigned short Port);

public:
  int   socketfd;               // The socket's file descriptor
  FILE *in, *out;               // File pointers to the above
  char  host[HOSTNAME_LENGTH];
  unsigned short port;         

    // used to create a server socket
  CoralSock(unsigned short Port);

    // used to create a client socket
  CoralSock(char *HostName, unsigned short Port = CORAL_PORT) {
    open_client(this, HostName, Port);  
  }

    // used to create a client (using environment variables or 
    //  defaults for server and port.)
  CoralSock();

 ~CoralSock() { 
      if (in)
        fclose(in);

      if (out)
        fclose(out);

      if (socketfd != -1) {
          shutdown(socketfd, 2);
          close(socketfd);
      }
  }

    // Check if a CoralSock is valid
  inline int Valid() { return(socketfd >= 0); }

    // Check if a CoralSock is valid
  inline int Local() { return(socketfd == 0); }

    // Convert socket to a file descriptor
  FILE *InFilePtr();
  FILE *OutFilePtr();

    // Accept (or refuse) a connection (server only)
  CoralSock *AcceptConnection();
  void RefuseConnection();
};


//**************************************************************************

#endif  // _CoralSock_H_
