// *************************************************************************
//    ParserEnv.h
// *************************************************************************

#ifndef _ParserEnv_H_
#define _ParserEnv_H_

#include <string.h>

#include "Literal.h"
#include "History.h"
#include "Config.h"
#include "Scanner.h"

// *************************************************************************
// An enumerated type to indicate where this ParserEnv is parsing from

enum ParseType {

    FromFile,            // From a file pointer
    FromBuff             // From a character buffer
};

// *************************************************************************

    //  LEXER is a pointer to a function returning int (usually yylex())
typedef int (*LEXER)(YYSTYPE *lvalp);

    // Redefined, this definition is provided by flex/lex
#ifndef YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

// *************************************************************************

class Cor_ParserEnv {

  public:
    int show_prompt;     // Should the prompt be shown after each command
    char prompt_name[100];
    char *prompt;

    Cor_History *history;
    int command_count;   // Used to number user commands for the history
                         // mechanism. If the default of -1 is not changed,
                         // the history mechanism is disabled. This is
                         // used by magic, and by imperative code. For the
                         // interpreter, it needs to be explicitly set to
                         // 1 before any commands are parsed
    int incr_command_count;

    int is_history_command; // Used to flag that the current command was a
                            // history command. In this case, the history
                            // stack should not be modified, since this
                            // is performed inside the historySolver in
                            // Solvers.C


    char linebuf[MAX_RULE_LENGTH];    // stores the current input line
    int  line_offset;

    char commandbuf[MAX_RULE_LENGTH]; // stores the current input command
    int  command_offset;


                              // These items are temp variables for use by
                              // the parser.  They were previously static
                              // and so needed to be moved here.
    Cor_BitVector CurTail;
    Cor_BitVector CurBoundVars;
    int           in_comment;

                              // Number of arithmetic literals introduced in
                              // converting arithmetic functions in the head
                              // literal.  These need to be moved to the end
                              // of the rule later

    int C_num_head_arith_lits;

    int complexRelation;      // used only for set grouping

    ParseType    ptype;       // Indicates which of the next 2 we're using
    LEXER        lexer;       // The lexer we are using [yylex() or sslex()]
    CoralStream *in_strm;     // A file pointer to parse from

    char    *parsebuff;       // A char buffer to parse from
    char    *parseptr;        // A pointer into the above buffer
    char    *parselimit;      // The end of the buffer we're parsing from

    YY_BUFFER_STATE yybuff;   // A buffer for the lexer

    char *filename;           // The name of this file
    int   cur_line_num;       // And its current line number

    Cor_ParserEnv     *next;  // For making a stack of ParserEnv's


    inline void set_prompt(int on) { show_prompt = (on ? 1 : 0); }

    // This is a hack to avoid the select problem that keeps coming
    // up, the client should reply with another command, or a newline
    // if no command is ready.
    void request_ack();

    void print_prompt();
    void handle_history();

      // Call after an error to repair input buffers
    void fix_history();

    void init();
    Cor_ParserEnv() { init();} ;
   ~Cor_ParserEnv();
};

// *************************************************************************

#endif _ParserEnv_H_
