// *************************************************************************
//    RMark.h
// *************************************************************************

#ifndef _RMark_H_
#define _RMark_H_

#include "TupleStack.h"
#include "Debug.h"

// *************************************************************************

class Cor_RelationLink;

// *************************************************************************
//  RMarks are used to select a sub-range of a relation.
//  The range (RM1, RM2) of relation R is the set of tuples in R that
//  were added to R after RM1 was created but before RM2 was created.
//  Either RM1 or RM2 can be NULL, which is equivalent to beginning/end 
//  of relation.  An RMark is created using 'relation->getMark()'.
// *************************************************************************

class Cor_RMark {

  public:
    Cor_RMark *prev;
    Cor_RMark *next;
    Cor_RelationLink *rel_p;          // new for purposes of delta_indexing
      // Given a startmark and an endmark, we consider each relation
      // in Relation.delta_relations (which is a RelationList)
      // starting from (and including) that associated with 
      // startmark.rel_p, up to (but not including) that associated
      // with endmark.rel_p

    Cor_TupleStack *tlist;         // The tuple list being referred to.
    int             tindex;        // The index to start at
      // The semantics of start and end marks is that tuples
      // after (but not including) the tuple indexed by startmark.tindex
      // in the TupleList are to be included, up to (and including) the
      // tuple indexed by endmark.tindex.
      // If (startmark.tindex == endmark.tindex), no tuples
      // are included.

    Cor_RMark();

    Cor_RMark(Cor_TupleStack *, int index);
    Cor_RMark(Cor_RelationLink *);

    void init(Cor_TupleStack *, int index);
    void init(Cor_RelationLink *);
};

// *************************************************************************

#endif _RMark_H_
