
% Data set for bom programs, the assembly hierarchy is a tree.

basic_part(spoke,1).
basic_part(tire,100).
basic_part(frame,500).
basic_part(bearing,15).
basic_part(axle,15).
basic_part(pedal,10).

%assembly(hub,bearing,2).
%assembly(hub,axle,1).
%assembly(wheel,spoke,25).
%assembly(wheel,tire,1).
%assembly(wheel,hub,1).
%assembly(bike,frame,1).
%assembly(bike,wheel,2).
%assembly(bike,pedal,2).

asyn_socket_rel(rem_assem(_,_,_), fontina, 6006, default_ws, assembly).

module bom_module.
export bom[bf,ff].

% This program computes the total cost of a composite part by summing
% the total costs of its subparts. This is often called the
% bill-of-materials problem. Note that the program is not stratified. 

@lazy_eval-.
@sup_magic-.
@magic+.
@async_rel rem_assem(X,Y,Z).


 % The assembly facts can be local
%assem(Part,Subpart,Quantity) :- assembly(Part,Subpart,Quantity).

 % or remote
%assem(Part,Subpart,Quantity) :- rem_assem(Part,Subpart,Quantity).

 % The rest is standard BOM
bom(Part,sum(<C>)) :- subpart_cost(Part,SubPart,C).
%bom(Part,C) :- subpart_cost(Part,SubPart,C).

subpart_cost(Part,Part,Cost) :- basic_part(Part,Cost).

subpart_cost(Part,Subpart,Cost) :- rem_assem(Part,Subpart,Quantity), 
	bom(Subpart, TotalSubcost),
	Cost = Quantity * TotalSubcost.

end_module.
