// *************************************************************************
//    Rule.h
// *************************************************************************

#ifndef _Rule_H_
#define _Rule_H_

#include "ClauseNames.h"
#include "Annotations.h"
#include "Stack.h"
#include "Entity.h"

// *************************************************************************

enum rule_type {
    COR_NORMALRULE,
    COR_MAGICRULE
};

// *************************************************************************

class Cor_Literal;
class Cor_Sips;

// *************************************************************************
// Rule structure created by the parser

class Cor_Clause {

  public:

    Cor_Clause *link;         // For grouping rules with the same head->pred
                              // together (for Collection's that have
                              // hashing turned on.)

    Cor_Clause *pnext;        // next, linked by predicate

    Cor_CArgStack *var_names; // List of all variable names used in the rule.
    int old_num_var_names;    // Used in Magic rewriting 

    Cor_Literal  *head;
    Cor_Literal **head_deletes;    // Array of delete literals in head
    int           head_deletes_count;

    int                     num_preds;
    Cor_Literal           **preds;     // List of rhs predicates
    Cor_Sips               *sips;
    struct Cor_RuleAnnotes *annotations; 

    int          last_part;      // last pred participating in sip
    int          line_number;    // source line of definition
    int          rule_number;    // unique rule number
    rule_type    type;           // COR_NORMAL/COR_MAGIC 
    Cor_Clause  *source_rule;    // For COR_MAGIC rule types only 
    Cor_Literal *source_pred;    // For COR_MAGIC rule types only
    unsigned int recursive; 
    unsigned int seen;           // flag used during scc analysis

    Cor_ClauseNames  *clause_names;

    int    factor_status;        // for factoring code only
    Cor_Literal *factor_pred;    // the rhs pred which is factorable
    int     ext_magic;           // external magic flag
    int     sn_rule_label_counter;
    int     supp;  // used in rewriting for nested_scc_eval ONLY.
                   // 0 for ordinary magic, 1 for sup_magic

#ifdef    REUSE_MEMORY
    void* operator new(size_t size) {
        return(Coral_Free->acquire(size));
    }

    void  operator delete(void *trash, size_t size) {
        Coral_Free->dispose((Cor_Ent_Element*) trash, size);
    }
#endif

    Cor_Clause();
   ~Cor_Clause();

      // Stream printing
    friend ostream& operator<< (ostream& out, const Cor_Clause& rule);
};

// *************************************************************************

#define FOR_EACH_CLAUSE_OF(rule, pred) \
  for (rule= (pred)->clauses; rule!=*(pred)->clause_tail; rule= (rule)->pnext)

void Cor_alloc_preds(Cor_Clause *rule, int count);
Cor_Clause *Cor_copy_rule(Cor_Clause *prototype);

// *************************************************************************

#endif _Rule_H_
