// *************************************************************************
//    History.C
// *************************************************************************

#include "History.h"

#include "Config.h"
#include "Globals.h"
#include "Debug.h"

// *************************************************************************
Cor_History::Cor_History(int size)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    commands = new char* [size];
    counts = new int [size];
    for (int i = 0; i < size; i++) {
        commands[i] = NULL;
        counts[i] = 0;
    }
    array_size = size;
    cur_pos = 0;
}

// *************************************************************************
Cor_History::Cor_History()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    array_size = MAX_HISTORY_DEPTH;
    commands = new char* [array_size];
    counts = new int [array_size];
    for (int i = 0; i < array_size; i++) {
        commands[i] = NULL;
        counts[i] = 0;
    }
    cur_pos = 0;
}

// *************************************************************************
Cor_History::~Cor_History()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int j;

    for (j=0; j < array_size; j++)
        delete commands[j];

    delete [] commands;
    delete [] counts;
}

// *************************************************************************
void Cor_History::addCommand(char *newcommand, int current_count)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    if (commands[cur_pos])
        delete (commands[cur_pos]);

    commands[cur_pos] = new char[strlen(newcommand) + 1];
    strcpy(commands[cur_pos], newcommand);
    counts[cur_pos] = current_count;
    if (++cur_pos >= array_size)
        cur_pos = 0;
}

// *************************************************************************
char *Cor_History::getCommand(int desired_count)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    for (int i = 0; i < array_size; i++) {
        if (counts[i] == desired_count)
            return commands[i];
    }
    return NULL;
}

// *************************************************************************
void Cor_History::display(ostream *outf)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int i;

    *outf << "    Command History\n";
    for (i = cur_pos; i < array_size; i++) {
        if (commands[i])
            *outf << "    " << counts[i] << " : " << commands[i] << endl;
    }

    for (i = 0; i < cur_pos; i++) {
        if (commands[i])
            *outf << "    " << counts[i] << " : " << commands[i] << endl;
    }

    *outf << endl;
}

// *************************************************************************
