// **************************************************************************
//    CoralStream.C
//
//    The CoralStream Class - A class to handle all I/O within Coral
//
// **************************************************************************

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <sys/types.h>

#ifdef    DO_PVM
#    include "PvmStream.h"
#else
#    include "SockStream.h"
#endif

#include "CoralStream.h"
#include "Debug.h"

// *************************************************************************
CoralStream::CoralStream(streambuf *tb, int delete_buffer)
: iostream(tb)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    orig = OST_NONE;
    _del_buffer = delete_buffer;
}

// *************************************************************************
CoralStream::~CoralStream()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    
    switch (orig) {

    default:
    case OST_NONE:
        if (_del_buffer)
            delete (this->rdbuf());
        break;

    case OST_IFS:
        delete ifs;
        break;

    case OST_OFS:
        delete ofs;
        break;
    }
}

// *************************************************************************
int   CoralStream::printf(char *fmt, ...)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    int ret;

    assert(this);

    va_list ap;
    va_start(ap,fmt);

    ret = rdbuf()->vform(fmt, ap);

    va_end(ap);
    return(ret);
}

// *************************************************************************
int   CoralStream::puts(char *buff, int add_nl)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    assert(this);
    *this << buff;
    if (add_nl)
        *this << endl;

    if (this->good())
        return(1);
    else
        return(EOF);
}

// *************************************************************************
int   CoralStream::putch(int ch)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    assert(this);
    char c = ch;
    *this << c;

    if (this->good())
        return(1);
    else
        return(EOF);
}

// *************************************************************************
//   Simulate what we need, neither getline() or get() quite do it.
int CoralStream::gets(char *buff, int n)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    assert(this);
    
    char c;
    int  tot = 0;

    n--;

    do {
        this->get(c);

        if (this->eof()) {
            break;
        }

       *buff++ = c;
        tot++;

    } while ((tot < n) && (c != '\n'));

   *buff = '\0';

    return(tot);
}

// *************************************************************************
int   CoralStream::scanf(char *fmt, ...)
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    assert(this);
    int ret;

    va_list ap;
    va_start(ap,fmt);

    ret = rdbuf()->vscan(fmt, ap);

    va_end(ap);
    return(ret);
}

// *************************************************************************
//#define    SERVER            1

// *************************************************************************
#ifdef TEST_STREAM

int main()
{
#ifdef    DO_CALL_STACK
    CallStk_Mark    callstk_mark(__FUNCTION__,__FILE__,__LINE__);
#endif
    //unsigned short port = 5432;
    unsigned short port = 7;    // TCP/IP echo port
    char result[200];

#    ifdef    SERVER

#    else

        // CLIENT

    Cor_sockbuf *echobuf = new Cor_sockbuf("quarg.cs.wisc.edu", port);

    CoralStream *in_cs = new CoralStream(cin.rdbuf());
    CoralStream *out_cs = new CoralStream(cout.rdbuf());
    CoralStream *echo_cs = new CoralStream(echobuf);

    int j=2;

        // Output possibilites (all should work)
    // *(out_cs) << "hello" << endl;
    out_cs->printf("hello%d\n", j);
    //out_cs->puts("hello");
    *(echo_cs) << "From socket" << endl;

    *out_cs << endl << flush;
    *echo_cs << flush;

        // Input possibilites (all should work)
    // *(in_cs) >> result;
    //in_cs->scanf("%s", result);
    //in_cs->gets(result,4);

    echo_cs->scanf("%s", result);

    cout << "Result is: " << result << endl;

#    endif  

    delete in_cs;
    delete out_cs;

    return(0);
}
#endif

// *************************************************************************
