/*****************************************************************************/
/*    Lexer.h   */
/*****************************************************************************/

#ifndef _Lexer_H_
#define _Lexer_H_

#include <stdio.h>
#include "Kind.h"

/*****************************************************************************/

#define MAX_CORAL_VARIABLES 100   /* per command */

/*****************************************************************************/
/*    Names for all tokens:   */

    /* Tokens should also be listed in Translate.C, in the tok_sym[] array */

typedef enum tok_type {

        EOF_TOK,

        SYMBOL,
        INUM,
        RNUM,
        VAR,
        STRING,      /* w/ quotes */
        EXPR,        /* What is this? */

        my_CORAL_BEGIN, my_CORAL_END, 
        my_INIT_CORAL, 
        my_EXIT_CORAL, 
        my_EXPORT_CORAL,

            /* Export Prototype Line */
        IDENT, SEMICOLON, 

        /* True Coral Tokens */

        APPEND, UPDATE, ASSIGN, DELETE, MARK,  /** 13 **/
        LPAREN, RPAREN, COMMA, COLON,
        SLASHSLASH, PERIOD, LBRACE, RBRACE,
        ARROW, SIPS, LBRACKET, RBRACKET,       /** 26 **/
        UNDERSCORE, VBAR, PERCENT,
        NOT, QUESTION, DOLLAR, COR_TRUE, SUM,
        DEL, START_C_MODULE, END_C_MODULE,
        MODULE, EXPORT, ALIAS, END_MODULE, CONSULT,
        PLUS, MINUS, STAR, SLASH,
        EQUAL, NOTEQUAL, LESS, GREATER, LESSEQUAL, GREATEREQUAL,
        LSHIFT, RSHIFT, BITXOR, BITAND, BITNOT, BITOR,
        MOD, AT, CUT, OP, RELOP,

            /* End-of-line when in embedded coral */
        EOLN,


            /*    Data types  */
        CHAR, INT, FLOAT, DOUBLE, VOID, CHAR_PTR, CHUNK, /** 66 **/
        CARG, CLASS, STRUCT,

            /*    optional qualifiers to ints  */
        SHORT, LONG, SIGNED, UNSIGNED,

            /* Misc */
        AS

} tok_type;

/*****************************************************************************/


typedef struct tok_st {
    int kind;
    union {
      int    inum;
      double rnum;
      char   *str;
    } val;
} Token;

        /*  For Flex compatibility   */
#define YYSTYPE Token

/*****************************************************************************/

#ifndef TTLEX_EXTRN

extern "C" int ttlex(YYSTYPE *lvalp);

extern "C" FILE *ttin;
extern "C" FILE *ttout;
extern "C" int   yylineno;

extern "C" void  line_directive(FILE *out);

#endif     /* TTLEX_EXTRN */
/*****************************************************************************/

#endif _Lexer_H_

